﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace NameCard {
    partial class FormEditCard : Form {
        private int[] _customcolors = null;

        public FormEditCard() {
            InitializeComponent();
        }

        protected override void OnClosing(CancelEventArgs e) {
            base.OnClosing(e);
            if(DialogResult == DialogResult.Cancel) return;

            e.Cancel = true;
            float dum;
            //Check width
            if(!float.TryParse(_txtWidth.Text, out dum)) {
                MessageBox.Show("幅の値が読み取れません。", "入力エラー",
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if(dum < 50) {
                if(MessageBox.Show("幅が50mmより小さいです。よろしいですか？", "入力注意",
                                    MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation,
                                    MessageBoxDefaultButton.Button2)
                   == DialogResult.No) return;
            }
            if(150 < dum) {
                if(MessageBox.Show("幅が150mmより大きいです。よろしいですか？", "入力注意",
                                    MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation,
                                    MessageBoxDefaultButton.Button2)
                   == DialogResult.No) return;
            }

            //Check height
            if(!float.TryParse(_txtHeight.Text, out dum)) {
                MessageBox.Show("高さの値が読み取れません。", "入力エラー",
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if(dum < 30) {
                if(MessageBox.Show("高さが30mmより小さいです。よろしいですか？", "入力注意",
                                    MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation,
                                    MessageBoxDefaultButton.Button2)
                   == DialogResult.No) return;
            }
            if(70 < dum) {
                if(MessageBox.Show("高さが70mmより大きいです。よろしいですか？", "入力注意",
                                    MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation,
                                    MessageBoxDefaultButton.Button2)
                   == DialogResult.No) return;
            }
            e.Cancel = false;
        }

        private void TextBox_Enter(object sender, EventArgs e) {
            ((TextBox)sender).SelectAll();
        }
        private void TextBox_KeyPress(object sender, KeyPressEventArgs e) {
            if('0' <= e.KeyChar && e.KeyChar <= '9') return;
            if(e.KeyChar == '.' && ((TextBox)sender).Text.IndexOf('.') < 0) return;
            if(e.KeyChar == '\b') return;
            //if(e.KeyChar == '\x18') return; // Ctrl+X;
            //if(e.KeyChar == '\x03') return; // Ctrl+C;
            //if(e.KeyChar == '\x16') return; // Ctrl+V;

            e.Handled = true;
        }
        private void _btnColor_Click(object sender, EventArgs e) {
            using(var dlg = new ColorDialog()) {
                dlg.AnyColor = true;
                dlg.Color = _btnColor.BackColor;
                dlg.CustomColors = _customcolors;
                if(DialogResult.Cancel == dlg.ShowDialog()) return;
                _btnColor.BackColor = dlg.Color;
                _customcolors = dlg.CustomColors;
            }
        }

        public SizeF Sizemm {
            get {
                return new SizeF(float.Parse(_txtWidth.Text),
                                 float.Parse(_txtHeight.Text));
            }
            set {
                _txtWidth.Text = value.Width.ToString();
                _txtHeight.Text = value.Height.ToString();
            }
        }
        public Color Color {
            get { return _btnColor.BackColor; }
            set { _btnColor.BackColor = value; }
        }
        public int[] CustomColors {
            get { return _customcolors; }
            set { _customcolors = value; }
        }
    }
}
