﻿using System;
using System.Drawing;
using System.Windows.Forms;

namespace NameCard {
    partial class FormEditChars : Form {
        private int[] _customcolors = null;
        public FormEditChars() {
            InitializeComponent();

            foreach(FontFamily ffm in FontFamily.Families)
                _cmbFontFamilyNames.Items.Add(ffm.Name);
        }

        protected override void OnClosing(System.ComponentModel.CancelEventArgs e) {
            base.OnClosing(e);
            if(DialogResult == DialogResult.Cancel) return;

            e.Cancel = true;
            if(_txtText.Text == null) {
                MessageBox.Show("文字列が入力されていません。", "入力エラー",
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if(_txtText.Text.Length == 0) {
                MessageBox.Show("文字列の長さがゼロです。", "入力エラー",
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            float dum;
            if(!float.TryParse(_txtFontSize.Text, out dum)) {
                MessageBox.Show("フォントサイズの値が読み取れません。", "入力エラー",
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if(dum < 2f) {
                MessageBox.Show("フォントサイズの値は2pt以上にして下さい。", "入力エラー",
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if(dum > 144f) {
                MessageBox.Show("フォントサイズの値は144pt以下にして下さい。", "入力エラー",
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if(!float.TryParse(_txtPositionLeft.Text, out dum)) {
                MessageBox.Show("位置左端の値が読み取れません。", "入力エラー",
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if(!float.TryParse(_txtPositionTop .Text, out dum)) {
                MessageBox.Show("位置上端の値が読み取れません。", "入力エラー",
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            e.Cancel = false;
        }

        private void TextBox_Enter(object sender, EventArgs e) {
            ((TextBox)sender).SelectAll();
        }
        private void TextBox_KeyPress(object sender, KeyPressEventArgs e) {
            if('0' <= e.KeyChar && e.KeyChar <= '9') return;
            if(e.KeyChar == '.' && ((TextBox)sender).Text.IndexOf('.') < 0) return;
            if(e.KeyChar == '\b') return;
            //if(e.KeyChar == '\x18') return; // Ctrl+X;
            //if(e.KeyChar == '\x03') return; // Ctrl+C;
            //if(e.KeyChar == '\x16') return; // Ctrl+V;

            e.Handled = true;
        }
        private void _btnFontColor_Click(object sender, EventArgs e) {
            using(var dlg = new ColorDialog()) {
                dlg.AnyColor = true;
                dlg.Color = _btnFontColor.BackColor;
                dlg.CustomColors = _customcolors;
                if(dlg.ShowDialog() == DialogResult.Cancel) return;
                _btnFontColor.BackColor = dlg.Color;
                _customcolors = dlg.CustomColors;
            }
        }
        private void _cmbFontFamilyNames_SelectedIndexChanged(object sender, EventArgs e) {
            _txtText.Font = new Font(CharsFont.FamilyName, 16, CharsFont.Style);
        }
        private void _chkFont_CheckedChanged(object sender, EventArgs e) {
            _txtText.Font = new Font(CharsFont.FamilyName, 16, CharsFont.Style);
        }
        private void ApplyFontToTextBox() {
            _txtText.Font = new Font(CharsFont.FamilyName, 16, CharsFont.Style);
        }

        public string CharsText {
            get { return _txtText.Text; }
            set { _txtText.Text = value; }
        }
        internal FontInfo CharsFont {
            get {
                FontInfo fi;
                fi.FamilyName = _cmbFontFamilyNames.Text;
                fi.EmSizePoint = float.Parse(_txtFontSize.Text);
                fi.Style = FontStyle.Regular;
                if(_chkFontBold.Checked) fi.Style |= FontStyle.Bold;
                if(_chkFontItalic.Checked) fi.Style |= FontStyle.Italic;
                if(_chkFontUnderline.Checked) fi.Style |= FontStyle.Underline;
                fi.Color = _btnFontColor.BackColor;
                return fi;
            }
            set {
                //FamilyName
                var i = _cmbFontFamilyNames.Items.IndexOf(value.FamilyName);
                if(i < 0) throw new System.Exception("The font is not available");
                _cmbFontFamilyNames.SelectedIndex = i;
                //EmSize
                _txtFontSize.Text = value.EmSizePoint.ToString();
                //Style
                _chkFontBold.Checked
                    = (value.Style & FontStyle.Bold) == FontStyle.Bold;
                _chkFontItalic.Checked
                    = (value.Style & FontStyle.Italic) == FontStyle.Italic;
                _chkFontUnderline.Checked
                    = (value.Style & FontStyle.Underline) == FontStyle.Underline;
                //Color
                _btnFontColor.BackColor = value.Color;
                _txtText.Font = new Font(CharsFont.FamilyName, 16, CharsFont.Style);
            }
        }
        public PointF CharsPosition {
            get {
                return new PointF(float.Parse(_txtPositionLeft.Text),
                                  float.Parse(_txtPositionTop.Text));
            }
            set {
                _txtPositionLeft.Text = value.X.ToString("0.00");
                _txtPositionTop.Text = value.Y.ToString("0.00");
            }
        }
        public int[] CustomColors {
            get { return _customcolors; }
            set { _customcolors = value; }
        }
    }
}
