﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace NameCard {
    partial class FormEditImage : Form {
        private const float MM_INCH = 25.4f;
        private const int MAX_WIDTH = 224, MAX_HEIGHT = 101;

        public FormEditImage() {
            InitializeComponent();
        }

        protected override void OnClosing(CancelEventArgs e) {
            base.OnClosing(e);
            if(DialogResult == DialogResult.Cancel) return;

            e.Cancel = true;
            if(_pctImage.Image == null) {
                MessageBox.Show("画像が選ばれていません。", "入力エラー",
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            float dum;
            if(!float.TryParse(_txtPositionLeft.Text, out dum)) {
                MessageBox.Show("位置左端の値が読み取れません。", "入力エラー",
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if(!float.TryParse(_txtPositionTop.Text, out dum)) {
                MessageBox.Show("位置上端の値が読み取れません。", "入力エラー",
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if(!float.TryParse(_txtWidth.Text, out dum)) {
                MessageBox.Show("サイズ幅の値が読み取れません。", "入力エラー",
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if(dum < 0.1) {
                MessageBox.Show("サイズ幅の値は0.1mm以上にして下さい。", "入力エラー",
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if(!float.TryParse(_txtHeight.Text, out dum)) {
                MessageBox.Show("サイズ高さの値が読み取れません。", "入力エラー",
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if(dum < 0.1) {
                MessageBox.Show("サイズ高さの値は0.1mm以上にして下さい。", "入力エラー",
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if(_pctImage.Image == null) {
                MessageBox.Show("画像が選ばれていません。", "入力エラー",
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            e.Cancel = false;
        }

        private void _btnFileRead_Click(object sender, EventArgs e) {
            Bitmap bmp;

            using(var dlg = new OpenFileDialog()) {
                dlg.CheckFileExists = true;
                dlg.CheckPathExists = true;
                dlg.Multiselect = false;
                if(DialogResult.Cancel == dlg.ShowDialog()) return;

                try {
                    bmp = new Bitmap(dlg.FileName);
                } catch(Exception ex) {
                    MessageBox.Show("画像ファイルの読み取りに失敗しました。\r\n\r\n"
                                  + ex.Message,
                                   "ファイルアクセスエラー",
                                   MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _pctImage.Image = null;
                    return;
                }
            }

            AdjustDrawSize(bmp);
            _pctImage.Image = bmp;
            _txtHeight.Text
                = ((float)bmp.Size.Height / bmp.VerticalResolution * MM_INCH).ToString("0.00");
            _txtWidth.Text
                = ((float)bmp.Size.Width / bmp.HorizontalResolution * MM_INCH).ToString("0.00");
        }
        private void TextBox_Enter(object sender, EventArgs e) {
            ((TextBox)sender).SelectAll();
        }
        private void TextBox_KeyPress(object sender, KeyPressEventArgs e) {
            if('0' <= e.KeyChar && e.KeyChar <= '9') return;
            if(e.KeyChar == '.' && ((TextBox)sender).Text.IndexOf('.') < 0) return;
            if(e.KeyChar == '\b') return;
            //if(e.KeyChar == '\x18') return; // Ctrl+X;
            //if(e.KeyChar == '\x03') return; // Ctrl+C;
            //if(e.KeyChar == '\x16') return; // Ctrl+V;

            e.Handled = true;
        }
        private void _txtWidth_TextChanged(object sender, EventArgs e) {
            if(_pctImage.Image == null) return;
            if(!_txtWidth.Focused) return;
            float w;
            if(!float.TryParse(_txtWidth.Text, out w)) return;
            _txtHeight.Text = (w * _pctImage.Image.Height / _pctImage.Image.Width).ToString("0.00");
        }
        private void _txtHeight_TextChanged(object sender, EventArgs e) {
            if(_pctImage.Image == null) return;
            if(!_txtHeight.Focused) return;
            float h;
            if(!float.TryParse(_txtHeight.Text, out h)) return;
            _txtWidth.Text = (h * _pctImage.Image.Width / _pctImage.Image.Height).ToString("0.00");
        }

        private void AdjustDrawSize(Bitmap bmp) {
            if(bmp.Width < MAX_WIDTH && bmp.Height < MAX_HEIGHT)
                _pctImage.Size = bmp.Size;
            else {
                var aspectbmp = (float)bmp.Width / bmp.Height;
                if(aspectbmp > (float)MAX_WIDTH / MAX_HEIGHT) {
                    _pctImage.Width = MAX_WIDTH;
                    _pctImage.Height = (int)(MAX_WIDTH / aspectbmp);
                } else {
                    _pctImage.Height = MAX_HEIGHT;
                    _pctImage.Width = (int)(MAX_HEIGHT * aspectbmp);
                }
            }
        }

        public Bitmap Image {
            get {
                return (Bitmap)_pctImage.Image;
            }
            set {
                AdjustDrawSize(value);
                _pctImage.Image = value;
                _btnFileRead.Enabled = false;
            }
        }
        public PointF ImagePosition {
            get {
                return new PointF(float.Parse(_txtPositionLeft.Text),
                                  float.Parse(_txtPositionTop.Text));
            }
            set {
                _txtPositionLeft.Text = value.X.ToString("0.00");
                _txtPositionTop.Text = value.Y.ToString("0.00");
            }
        }
        public SizeF ImageSize {
            get {
                return new SizeF(float.Parse(_txtWidth.Text),
                                 float.Parse(_txtHeight.Text));
            }
            set {
                _txtWidth.Text = value.Width.ToString("0.00");
                _txtHeight.Text = value.Height.ToString("0.00");
            }
        }
    }
}
