﻿using System;
using System.ComponentModel;
using System.Drawing.Printing;
using System.Windows.Forms;

namespace NameCard {
    partial class FormEditSheet : Form {
        public FormEditSheet() {
            InitializeComponent();
            for(var i = 1; i <= 118; i++) {
                if(i == 48 || i == 49) continue;
                _cmbSheetSize.Items.Add((PaperKind)i);
            }
            _cmbSheetSize.SelectedItem = PaperKind.A4;
        }

        protected override void OnClosing(CancelEventArgs e) {
            base.OnClosing(e);
            if(DialogResult == DialogResult.Cancel) return;

            e.Cancel = true;
            float fdum;
            if(!float.TryParse(_txtMarginLeft.Text, out fdum)) {
                MessageBox.Show("印刷開始位置（紙左端から）の値が読み取れません。", "入力エラー",
                                 MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if(!float.TryParse(_txtMarginTop.Text, out fdum)) {
                MessageBox.Show("印刷開始位置（上端から）の値が読み取れません。", "入力エラー",
                                 MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            int idum;
            if(!int.TryParse(_txtM.Text, out idum)) {
                MessageBox.Show("枚数（横）の値が読み取れません。", "入力エラー",
                                 MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if(idum == 0) {
                MessageBox.Show("枚数（横）の値は1以上にして下さい。", "入力エラー",
                           MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if(!int.TryParse(_txtN.Text, out idum)) {
                MessageBox.Show("枚数（縦）の値が読み取れません。", "入力エラー",
                                 MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if(idum == 0) {
                MessageBox.Show("枚数（縦）の値は1以上にして下さい。", "入力エラー",
                           MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if(!float.TryParse(_txtIntervalH.Text, out fdum)) {
                MessageBox.Show("インターバル（横）の値が読み取れません。", "入力エラー",
                                 MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if(!float.TryParse(_txtIntervalV.Text, out fdum)) {
                MessageBox.Show("インターバル（縦）の値が読み取れません。", "入力エラー",
                                 MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            e.Cancel = false;
        }
        private void TextBox_Enter(object sender, EventArgs e) {
            ((TextBox)sender).SelectAll();
        }
        private void TextBox_KeyPress(object sender, KeyPressEventArgs e) {
            if('0' <= e.KeyChar && e.KeyChar <= '9') return;
            if(e.KeyChar == '\b') return;
            var tb = (TextBox)sender;
            if(tb != _txtM && tb != _txtN)
                if(e.KeyChar == '.' && tb.Text.IndexOf('.') < 0) return;
            e.Handled = true;
        }

        public PaperKind Size {
            get { return (PaperKind)_cmbSheetSize.SelectedItem; }
            set { _cmbSheetSize.SelectedItem = value; }
        }
        public float MarginLeft {
            get { return float.Parse(_txtMarginLeft.Text); }
            set { _txtMarginLeft.Text = value.ToString("0.00"); }
        }
        public float MarginTop {
            get { return float.Parse(_txtMarginTop.Text); }
            set { _txtMarginTop.Text = value.ToString("0.00"); }
        }
        public int M {
            get { return int.Parse(_txtM.Text); }
            set { _txtM.Text = value.ToString(); }
        }
        public int N {
            get { return int.Parse(_txtN.Text); }
            set { _txtN.Text = value.ToString(); }
        }
        public float IntervalH {
            get { return float.Parse(_txtIntervalH.Text); }
            set { _txtIntervalH.Text = value.ToString("0.00"); }
        }
        public float IntervalV {
            get { return float.Parse(_txtIntervalV.Text); }
            set { _txtIntervalV.Text = value.ToString("0.00"); }
        }
    }
}
