﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Printing;
using System.IO;
using System.Reflection;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Windows.Forms;

namespace NameCard {
    partial class FormMain : Form {
        private const int WM_KEYDOWN = 0x0100;
        private const int WM_KEYUP = 0x0101;
        private const int WM_CHAR = 0x0102;
        private const int WM_SYSKEYDOWN = 0x0104;
        private const int WM_SYSKEYUP = 0x0105;
        private const int KEY_REPEAT = 0x40000000;

        private const float MM_INCH = 25.4f;
        private const float MM_INCH100TH = MM_INCH / 100;

        private const string FILEDROP = "FileDrop";

        private int _lastxp = -1, _lastyp = -1;
        private float _dpix, _dpiy, _scale;
        private bool _keyrepeat = false;
        private Properties.Settings _setting = new Properties.Settings();

        private Editor _card = null;
        private SheetLayout _sheetlayout = new SheetLayout();
        private Dictionary<string, ToolStripItem> _contextmenus
            = new Dictionary<string, ToolStripItem>();

        public FormMain() {
            //Generated by IDE
            InitializeComponent();
            //Try to find an old-version setting
            if(_setting.NameCardFilePath.Length == 0) _setting.Upgrade();
            //Get dpis of screen
            using(var g = this.CreateGraphics()) { _dpix = g.DpiX; _dpiy = g.DpiY; }
            //Set up the menu of display magnifier
            foreach(var s in MagnifyingFactors.Strings) _tscMagnify.Items.Add(s);
            _tscMagnify.SelectedItem = MagnifyingFactors.DefaultString;
            //Set up the menu of move stride
            foreach(var s in MoveStrides.Strings) _tscStride.Items.Add(s);
            _tscStride.SelectedItem = MoveStrides.DefaultString;
            //Set up the contextmenus
            AddToContextMenus(_tsmEditCard.Text, null, _tsmEditCard_Click);
            AddToContextMenus(_tsmSelectAll.Text, null, _tsmSelectAll_Click);
            AddToContextMenus(_tsmInsertCharacters.Text, null, _tsmInsertCharacters_Click);
            AddToContextMenus(_tsmEditCharacters.Text, null, _tsmEditCharacters_Click);
            AddToContextMenus(_tsmInsertImage.Text, null, _tsmInsertImage_Click);
            AddToContextMenus(_tsmEditImage.Text, null, _tsmEditImage_Click);
            AddToContextMenus(_tsmAlignLeft.Text, null, _tsmAlign_Click);
            AddToContextMenus(_tsmAlignCenter.Text, null, _tsmAlign_Click);
            AddToContextMenus(_tsmAlignRight.Text, null, _tsmAlign_Click);
            AddToContextMenus(_tsmAlignUp.Text, null, _tsmAlign_Click);
            AddToContextMenus(_tsmAlignMiddle.Text, null, _tsmAlign_Click);
            AddToContextMenus(_tsmAlignDown.Text, null, _tsmAlign_Click);
            AddToContextMenus(_tsmAlignEqualGap.Text, null, _tsmAlign_Click);
            AddToContextMenus(_tsmAlignNoGap.Text, null, _tsmAlign_Click);
            AddToContextMenus(_tsmAlignFontInterval.Text, null, _tsmAlign_Click);
            AddToContextMenus(_tsmGroup.Text, null, _tsmGroup_Click);
            AddToContextMenus(_tsmDissolve.Text, null, _tsmGroup_Click);
            AddToContextMenus(_tsmDelete.Text, null, _tsmDelete_Click);
            //Configure on-off of menus
            EnableMenusInitial();
        }
        private void AddToContextMenus(string text, Image img, EventHandler onclick) {
            var tsmi = new ToolStripMenuItem(text, img, onclick);
            _contextmenus.Add(tsmi.Text, tsmi);
        }

        public override bool PreProcessMessage(ref Message msg) {
            if(msg.Msg == WM_KEYDOWN)
                _keyrepeat = ((int)msg.LParam & KEY_REPEAT) != 0;
            else if(msg.Msg == WM_KEYUP)
                _keyrepeat = false; ;
            return base.PreProcessMessage(ref msg);
        }
        protected override void OnDragOver(DragEventArgs e) {
            e.Effect = DragDropEffects.None;
            if(!e.Data.GetDataPresent(FILEDROP)) return;
            var filenames = (string[])e.Data.GetData(FILEDROP);
            if(filenames.Length > 1) return;
            var file = new FileInfo(filenames[0]);
            if((file.Attributes & FileAttributes.Directory) > 0) return;
            if(!filenames[0].ToLower().EndsWith(".nmc")) return;
            e.Effect = DragDropEffects.Link;
        }
        protected override void OnDragDrop(DragEventArgs e) {
            var filenames = (string[])e.Data.GetData(FILEDROP);
            FileOpen(filenames[0]);
        }
        protected override void OnMouseDown(MouseEventArgs e) {
            base.OnMouseDown(e);
            if(_card == null) return;
            if(e.Button == MouseButtons.Left) {
                _lastxp = _lastyp = -1;
                _card.Hit(new PointF(Xmm(e.X), Ymm(e.Y)),
                          Control.ModifierKeys == Keys.Control);
                EnableEditMenus(_card.SelectionStatus);
            } else if(e.Button == MouseButtons.Right) {
                _lastxp = e.X; _lastyp = e.Y;
                _cmsMain.Show(this.PointToScreen(e.Location));
            }
        }
        private float Ymm(float ypx) {
            return (ypx - _msMain.Height) / _dpiy * MM_INCH / _scale;
        }
        private float Xmm(float xpx) {
            return xpx / _dpix * MM_INCH / _scale;
        }
        protected override void OnPaint(PaintEventArgs e) {
            base.OnPaint(e);
            if(_card == null) return;
            PaintCard(e.Graphics);
        }
        private void PaintCard(Graphics g) {
            g.PageUnit = GraphicsUnit.Millimeter;
            g.ScaleTransform(_scale, _scale);
            g.TranslateTransform(0f, -Ymm(0));
            _card.BeDrawnOn(g);
        }
        protected override void OnClosing(CancelEventArgs e) {
            base.OnClosing(e);
            _setting.Save();
            if(_card == null) return;
            if(_tsmUndo.Enabled) {
                if(DialogResult.Cancel == MessageBoxConfirmDispose())
                    e.Cancel = true;
            } else {
                _card.Dispose();
            }
        }

        private void _cmsMain_Opening(object sender, CancelEventArgs e) {
            if(_card.SelectionStatus == GISelectionStatus.NoItemFirstOfAll ||
               _card.SelectionStatus == GISelectionStatus.None) {
                _cmsMain.Items.Add(_contextmenus[_tsmEditCard.Text]);
                _cmsMain.Items.Add(_contextmenus[_tsmInsertCharacters.Text]);
                _cmsMain.Items.Add(_contextmenus[_tsmInsertImage.Text]);
                if(_card.SelectionStatus == GISelectionStatus.None)
                    _cmsMain.Items.Insert(0, _contextmenus[_tsmSelectAll.Text]);

            } else {
                _cmsMain.Items.Add(_contextmenus[_tsmAlignLeft.Text]);
                _cmsMain.Items.Add(_contextmenus[_tsmAlignCenter.Text]);
                _cmsMain.Items.Add(_contextmenus[_tsmAlignRight.Text]);
                _cmsMain.Items.Add(_contextmenus[_tsmAlignUp.Text]);
                _cmsMain.Items.Add(_contextmenus[_tsmAlignMiddle.Text]);
                _cmsMain.Items.Add(_contextmenus[_tsmAlignDown.Text]);
                if(_card.SelectionStatus == GISelectionStatus.OneSetOfCharacters)
                    _cmsMain.Items.Insert(0, _contextmenus[_tsmEditCharacters.Text]);
                else if(_card.SelectionStatus == GISelectionStatus.OneBitmapImage)
                    _cmsMain.Items.Insert(0, _contextmenus[_tsmEditImage.Text]);
                else if(_card.SelectionStatus == GISelectionStatus.OneGroupedItem)
                    _cmsMain.Items.Insert(0, _contextmenus[_tsmDissolve.Text]);
                else {
                    _cmsMain.Items.Add(_contextmenus[_tsmAlignEqualGap.Text]);
                    _cmsMain.Items.Add(_contextmenus[_tsmAlignNoGap.Text]);
                    _cmsMain.Items.Add(_contextmenus[_tsmAlignFontInterval.Text]);
                    _cmsMain.Items.Add(_contextmenus[_tsmGroup.Text]);
                }
                _cmsMain.Items.Add(_contextmenus[_tsmDelete.Text]);
            }
            e.Cancel = false;
        }
        private void _cmsMain_Closed(object sender, ToolStripDropDownClosedEventArgs e) {
            _cmsMain.Items.Clear();
        }

        private void _tsmNew_Click(object sender, EventArgs e) {
            _card = new Editor(this);
            EnableMenusOnLoad();
        }
        private void _tsmOpen_Click(object sender, EventArgs e) {
            using(var dlg = new OpenFileDialog()) {
                dlg.InitialDirectory = _setting.NameCardFilePath.Length == 0 ?
                    Environment.GetFolderPath(Environment.SpecialFolder.Personal) :
                    _setting.NameCardFilePath;
                dlg.Filter = Editor.FileFilter;
                dlg.CheckPathExists = true;
                dlg.CheckFileExists = true;
                dlg.RestoreDirectory = true;
                if(DialogResult.Cancel == dlg.ShowDialog()) return;
                if(_card != null)
                    if(_tsmUndo.Enabled) {
                        if(DialogResult.Cancel == MessageBoxConfirmDispose())
                            return;
                    } else
                        _card.Dispose();
                FileOpen(dlg.FileName);
            }
        }
        private void FileOpen(string filename) {
            try {
                _card = new Editor(this, new FileInfo(filename));
            } catch(IOException ex) {
                TerminateFileOpening();
                MessageBox.Show("ファイルの読み取りに失敗しました。\r\n\r\n" + ex.Message,
                                "IOException",
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            } catch(SerializationException ex) {
                TerminateFileOpening();
                MessageBox.Show("名刺データではありません。\r\n\r\n" + ex.Message,
                                "SerializationException",
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            try {
                using(var g = this.CreateGraphics()) PaintCard(g);
            } catch(ArgumentException ex) {
                TerminateFileOpening();
                MessageBox.Show("読み取ったデータで描画できません。\r\n\r\n" + ex.Message,
                                "ArgumentException",
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _setting.NameCardFilePath = _card.File.Directory.FullName;
            EnableMenusOnLoad();
        }
        private void TerminateFileOpening() {
            _card = null;
            this.Invalidate();
            EnableMenusInitial();
        }
        private void _tsmSave_Click(object sender, EventArgs e) {
            if(_card == null) return;
            if(_card.File != null) {
                try {
                    _card.Save();
                } catch(IOException ex) {
                    MessageBox.Show("ファイルの上書きができません。\r\n\r\n"
                                    + ex.Message,
                                    "IOException",
                                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            } else
                SaveAs();
        }
        private void _tsmSaveAs_Click(object sender, EventArgs e) {
            if(_card == null) return;
            SaveAs();
        }
        private void SaveAs() {
            using(var dlg = new SaveFileDialog()) {
                if(_card.File != null) dlg.FileName = _card.File.Name;
                dlg.Filter = Editor.FileFilter;
                dlg.OverwritePrompt = true;
                dlg.CreatePrompt = false;
                dlg.AddExtension = true;
                dlg.RestoreDirectory = true;
                if(DialogResult.Cancel == dlg.ShowDialog()) return;
                try {
                    _card.SaveAs(new FileInfo(dlg.FileName));
                } catch(IOException ex) {
                    MessageBox.Show("ファイルに保存できません。\r\n\r\n"
                                    + ex.Message,
                                    "IOException",
                                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            _setting.NameCardFilePath = _card.File.Directory.FullName;
        }
        private void _tsmQuit_Click(object sender, EventArgs e) {
            this.Close();
        }
        private DialogResult MessageBoxConfirmDispose() {
            return MessageBox.Show("作業中のデータがあります。破棄しますか？",
                                   "データ喪失注意",
                                   MessageBoxButtons.OKCancel,
                                   MessageBoxIcon.Exclamation,
                                   MessageBoxDefaultButton.Button2);
        }
        private void EnableMenusInitial() {
            _tsmFile.Enabled = true;
            _tsmNew.Enabled = true;
            _tsmOpen.Enabled = true;
            _tsmSave.Enabled = false;
            _tsmSaveAs.Enabled = false;
            _tsmQuit.Enabled = true;

            _tsmEdit.Enabled = false;
            _tsmUndo.Enabled = false;
            _tsmEditCard.Enabled = false;
            _tsmInsertCharacters.Enabled = false;
            _tsmInsertImage.Enabled = false;

            _tsmLayout.Enabled = false;
            _tscStride.Enabled = false;

            _tsmPrint.Enabled = false;

            EnableEditMenus(GISelectionStatus.NoItemFirstOfAll);
        }
        private void EnableMenusOnLoad() {
            _tsmSave.Enabled = true;
            _tsmSaveAs.Enabled = true;

            _tsmEdit.Enabled = true;
            _tsmUndo.Enabled = false;
            _tsmEditCard.Enabled = true;
            _tsmInsertCharacters.Enabled = true;
            _tsmInsertImage.Enabled = true;

            _tsmLayout.Enabled = true;
            _tscStride.Enabled = true;

            _tsmPrint.Enabled = true;

            EnableEditMenus(_card.SelectionStatus);
        }
        private void EnableEditMenus(GISelectionStatus selstat) {
            _tsmDelete.Enabled = false;
            _tsmEditCharacters.Enabled = false;
            _tsmEditImage.Enabled = false;

            _tsmCopyPaste.Enabled = false;
            _tsmSelectAll.Enabled = false;
            _tsmGroup.Enabled = false;
            _tsmDissolve.Enabled = false;

            _tsmMoveRight.Enabled = false;
            _tsmMoveLeft.Enabled = false;
            _tsmMoveUp.Enabled = false;
            _tsmMoveDown.Enabled = false;

            _tsmAlignLeft.Enabled = false;
            _tsmAlignRight.Enabled = false;
            _tsmAlignCenter.Enabled = false;
            _tsmAlignUp.Enabled = false;
            _tsmAlignMiddle.Enabled = false;
            _tsmAlignCenter.Enabled = false;
            _tsmAlignDown.Enabled = false;
            _tsmAlignEqualGap.Enabled = false;
            _tsmAlignNoGap.Enabled = false;
            _tsmAlignFontInterval.Enabled = false;

            if(selstat == GISelectionStatus.NoItemFirstOfAll) return;

            _tsmSelectAll.Enabled = true;

            if(selstat == GISelectionStatus.None) return;

            _tsmDelete.Enabled = true;
            _tsmMoveRight.Enabled = true;
            _tsmMoveLeft.Enabled = true;
            _tsmMoveUp.Enabled = true;
            _tsmMoveDown.Enabled = true;

            if(selstat == GISelectionStatus.MultipleItems) {
                _tsmAlignLeft.Enabled = true;
                _tsmAlignRight.Enabled = true;
                _tsmAlignCenter.Enabled = true;
                _tsmAlignUp.Enabled = true;
                _tsmAlignMiddle.Enabled = true;
                _tsmAlignDown.Enabled = true;
                _tsmAlignEqualGap.Enabled = true;
                _tsmAlignNoGap.Enabled = true;
                _tsmAlignFontInterval.Enabled = true;
                _tsmGroup.Enabled = true;
                return;
            }

            _tsmCopyPaste.Enabled = true;
            _tsmAlignLeft.Enabled = true;
            _tsmAlignRight.Enabled = true;
            _tsmAlignCenter.Enabled = true;
            _tsmAlignUp.Enabled = true;
            _tsmAlignMiddle.Enabled = true;
            _tsmAlignDown.Enabled = true;

            if(selstat == GISelectionStatus.OneBitmapImage)
                _tsmEditImage.Enabled = true;
            else if(selstat == GISelectionStatus.OneSetOfCharacters)
                _tsmEditCharacters.Enabled = true;
            else
                _tsmDissolve.Enabled = true;
        }
        public void EnableUndo() {
            _tsmUndo.Enabled = _card.UndoListCount != 0;
        }

        private void _tsmUndo_Click(object sender, EventArgs e) {
            _card.PopFromUndoList();
        }
        private void _tsmEditCard_Click(object sender, EventArgs e) {
            using(FormEditCard dlg = new FormEditCard()) {
                dlg.Sizemm = _card.Sizemm;
                dlg.Color = _card.Color;
                dlg.CustomColors = _card.CustomColors;
                if(dlg.ShowDialog() == DialogResult.Cancel) return;
                _card.Sizemm = dlg.Sizemm;
                _card.Color = dlg.Color;
                _card.CustomColors = dlg.CustomColors;
            }
        }
        private void _tsmSelectAll_Click(object sender, EventArgs e) {
            EnableEditMenus(_card.SelectAll());
        }
        private void _tsmCopyPaste_Click(object sender, EventArgs e) {
            EnableEditMenus(_card.CopyPaste());
        }
        private void _tsmInsertCharacters_Click(object sender, EventArgs e) {
            using(var dlg = new FormEditChars()) {
                dlg.CharsFont
                    = new FontInfo("ＭＳ 明朝", 9.0f, FontStyle.Regular, Color.Black);
                dlg.CharsText = "新規文字";
                dlg.CharsPosition = _lastxp < 0 ?
                                    new PointF(0.0f, 0.0f) :
                                    new PointF(Xmm(_lastxp), Ymm(_lastyp));
                dlg.CustomColors = _card.CustomColors;
                if(dlg.ShowDialog() == DialogResult.Cancel) return;
                EnableEditMenus(_card.InsertCharacters(dlg.CharsText,
                                                       dlg.CharsFont,
                                                       dlg.CharsPosition));
                _card.CustomColors = dlg.CustomColors;
            }
        }
        private void _tsmEditCharacters_Click(object sender, EventArgs e) {
            string text; FontInfo font; PointF posmm;
            _card.GetSelectedCharacters(out text, out font, out posmm);
            using(FormEditChars dlg = new FormEditChars()) {
                dlg.CharsText = text;
                dlg.CharsFont = font;
                dlg.CharsPosition = posmm;
                dlg.CustomColors = _card.CustomColors;
                if(dlg.ShowDialog() == DialogResult.Cancel) return;
                _card.EditCharacters(dlg.CharsText,
                                     dlg.CharsFont,
                                     dlg.CharsPosition);
                _card.CustomColors = dlg.CustomColors;
            }
        }
        private void _tsmInsertImage_Click(object sender, EventArgs e) {
            using(var dlg = new FormEditImage()) {
                dlg.ImagePosition = _lastxp < 0 ?
                                    new PointF(0.0f, 0.0f) :
                                    new PointF(Xmm(_lastxp), Ymm(_lastyp));
                dlg.ImageSize = new SizeF(0, 0);
                if(dlg.ShowDialog() == DialogResult.Cancel) return;
                EnableEditMenus(_card.InsertImage(dlg.Image,
                                                  dlg.ImageSize,
                                                  dlg.ImagePosition));
            }
        }
        private void _tsmEditImage_Click(object sender, EventArgs e) {
            Bitmap image; SizeF sizemm; PointF posmm;
            _card.GetSelectedImage(out image, out sizemm, out posmm);
            using(var dlg = new FormEditImage()) {
                dlg.Image = image;
                dlg.ImageSize = sizemm;
                dlg.ImagePosition = posmm; ;
                if(dlg.ShowDialog() == DialogResult.OK)
                    _card.EditImage(dlg.ImageSize, dlg.ImagePosition);
            }
            image.Dispose();
        }
        private void _tsmGroup_Click(object sender, EventArgs e) {
            var txt = ((ToolStripMenuItem)sender).Text;
            EnableEditMenus(_tsmGroup.Text == txt ?
                            _card.Group() : _card.Dissolve());
        }
        private void _tsmDelete_Click(object sender, EventArgs e) {
            EnableEditMenus(_card.Remove());
        }

        private class MoveStrides {
            private static float[] _strides = { 0.1f, 0.5f, 1f };
            private const int DEFAULT_INDEX = 1;
            private static string[] _strings;
            static MoveStrides() {
                _strings = new string[_strides.Length];
                for(var i = 0; i < _strides.Length; i++)
                    _strings[i] = _strides[i].ToString("移動量0.0mm");
            }
            public static IEnumerable<string> Strings {
                get { foreach(string s in _strings) yield return s; }
            }
            public static float GetStride(string s) {
                var i = Array.IndexOf(_strings, s);
                if(i < 0) throw new ArgumentNullException("The string is not found in the list.");
                return _strides[i];
            }
            public static float DefaultStride { get { return _strides[DEFAULT_INDEX]; } }
            public static string DefaultString { get { return _strings[DEFAULT_INDEX]; } }
        }
        private void _tsmMove_Click(object sender, EventArgs e) {
            var len = MoveStrides.GetStride((string)_tscStride.SelectedItem);
            if(sender == _tsmMoveLeft)
                _card.Move(Editor.Direction.Left, len, _keyrepeat);
            else if(sender == _tsmMoveRight)
                _card.Move(Editor.Direction.Right, len, _keyrepeat);
            else if(sender == _tsmMoveUp)
                _card.Move(Editor.Direction.Up, len, _keyrepeat);
            else if(sender == _tsmMoveDown)
                _card.Move(Editor.Direction.Down, len, _keyrepeat);
            this.Update();
        }
        private void _tsmAlign_Click(object sender, EventArgs e) {
            var txt = ((ToolStripMenuItem)sender).Text;
            if(_tsmAlignLeft.Text == txt)
                _card.Align(Editor.Direction.Left);
            else if(_tsmAlignCenter.Text == txt)
                _card.Align(Editor.Direction.Center);
            else if(_tsmAlignRight.Text == txt)
                _card.Align(Editor.Direction.Right);
            else if(_tsmAlignUp.Text == txt)
                _card.Align(Editor.Direction.Up);
            else if(_tsmAlignMiddle.Text == txt)
                _card.Align(Editor.Direction.Middle);
            else if(_tsmAlignDown.Text == txt)
                _card.Align(Editor.Direction.Down);
            else if(_tsmAlignEqualGap.Text == txt)
                _card.AlignEqualGap();
            else if(_tsmAlignNoGap.Text == txt)
                _card.AlignNoGap();
            else if(_tsmAlignFontInterval.Text == txt)
                _card.AlignFontInterval();
        }

        private void _tscMagnify_SelectedIndexChanged(object sender, EventArgs e) {
            _scale = MagnifyingFactors.GetFactor((string)_tscMagnify.SelectedItem);
            this.Invalidate();
        }
        private class MagnifyingFactors {
            private static float[] _factors = { 0.75f, 1f, 1.5f, 2f };
            private const int DEFAULT_INDEX = 1;
            private static string[] _strings;
            static MagnifyingFactors() {
                _strings = new string[_factors.Length];
                for(var i = 0; i < _factors.Length; i++)
                    _strings[i] = _factors[i].ToString("倍率#0%");
            }
            public static IEnumerable<string> Strings {
                get { foreach(string s in _strings) yield return s; }
            }
            public static float GetFactor(string s) {
                var i = Array.IndexOf(_strings, s);
                if(i < 0) throw new System.ArgumentNullException("The string is not found in the list.");
                return _factors[i];
            }
            public static float DefaultFactor { get { return _factors[DEFAULT_INDEX]; } }
            public static string DefaultString { get { return _strings[DEFAULT_INDEX]; } }
        }

        private void _tsmPrintACard_Click(object sender, EventArgs e) {
            if(_card == null) return;
            using(var dlg = new PrintDialog()) {
                dlg.UseEXDialog = true;
                dlg.Document = _prtDocument;
                if(DialogResult.Cancel == dlg.ShowDialog()) return;
            }
            _prtDocument.OriginAtMargins = false;
            _prtDocument.PrintPage += new PrintPageEventHandler(_prtDocument_PrintACard);
            _prtDocument.Print();
        }
        private void _prtDocument_PrintACard(object sender, PrintPageEventArgs e) {
            e.Graphics.PageUnit = GraphicsUnit.Millimeter;
            PrintACardWithAFrame(e);
            e.HasMorePages = false;
            _prtDocument.PrintPage -= new PrintPageEventHandler(_prtDocument_PrintACard);
        }
        private void _tsmPrintASheet_Click(object sender, EventArgs e) {
            if(_card == null) return;
            using(var dlg = new FormEditSheet()) {
                dlg.Size = _sheetlayout.Size.Kind;
                dlg.MarginLeft = _sheetlayout.MarginLeftmm;
                dlg.MarginTop = _sheetlayout.MarginTopmm;
                dlg.M = _sheetlayout.M;
                dlg.N = _sheetlayout.N;
                dlg.IntervalH = _sheetlayout.IntervalHmm;
                dlg.IntervalV = _sheetlayout.IntervalVmm;
                if(DialogResult.Cancel == dlg.ShowDialog()) return;
                _sheetlayout.Size.RawKind = (int)dlg.Size;
                _sheetlayout.MarginLeftmm = dlg.MarginLeft;
                _sheetlayout.MarginTopmm = dlg.MarginTop;
                _sheetlayout.M = dlg.M;
                _sheetlayout.N = dlg.N;
                _sheetlayout.IntervalHmm = dlg.IntervalH;
                _sheetlayout.IntervalVmm = dlg.IntervalV;
            }
            _prtDocument.DefaultPageSettings.PaperSize = _sheetlayout.Size;
            using(var dlg = new PrintDialog()) {
                dlg.UseEXDialog = true;
                dlg.Document = _prtDocument;
                if(DialogResult.Cancel == dlg.ShowDialog()) return;
            }
            _prtDocument.OriginAtMargins = false;
            _prtDocument.PrintPage += new PrintPageEventHandler(_prtDocument_PrintASheet);
            _prtDocument.Print();
        }
        private void _prtDocument_PrintASheet(object sender, PrintPageEventArgs e) {
            e.Graphics.PageUnit = GraphicsUnit.Millimeter;
            for(var i = 0; i < _sheetlayout.M; i++)
                for(var j = 0; j < _sheetlayout.N; j++) {
                    SetTranslation(i, j, e);
                    _card.BePrintedOn(e.Graphics);
                    using(var pen = new Pen(_card.NegativeColor, 0))
                        e.Graphics.DrawRectangle(pen, 0, 0, _card.Sizemm.Width, _card.Sizemm.Height);
                }
            e.HasMorePages = false;
            _prtDocument.PrintPage -= new PrintPageEventHandler(_prtDocument_PrintASheet);
        }
        private void _tsmSheetLayoutSave_Click(object sender, EventArgs e) {
            FileInfo file;
            using(var dlg = new SaveFileDialog()) {
                if(_sheetlayout.File != null) dlg.FileName = _sheetlayout.File.Name;
                dlg.Filter = SheetLayout.FileFilter;
                dlg.OverwritePrompt = true;
                dlg.CreatePrompt = false;
                dlg.AddExtension = true;
                if(DialogResult.Cancel == dlg.ShowDialog()) return;
                file = new FileInfo(dlg.FileName);
            }
            try {
                var formatter = new BinaryFormatter();
                using(var s = file.OpenWrite())
                    formatter.Serialize(s, _sheetlayout);
            } catch(IOException ex) {
                MessageBox.Show("ファイルに保存できません。\r\n\r\n"
                                + ex.Message,
                                "IOException",
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            _sheetlayout.File = file;
            _setting.SheetLayoutFilePath = file.Directory.FullName;
        }
        private void _tsmSheetLayoutOpen_Click(object sender, EventArgs e) {
            FileInfo file;
            using(var dlg = new OpenFileDialog()) {
                dlg.InitialDirectory = _setting.SheetLayoutFilePath.Length == 0 ?
                    Environment.GetFolderPath(Environment.SpecialFolder.Personal) :
                    _setting.SheetLayoutFilePath;
                dlg.Filter = SheetLayout.FileFilter;
                dlg.CheckPathExists = true;
                dlg.CheckFileExists = true;
                dlg.RestoreDirectory = true;
                if(DialogResult.Cancel == dlg.ShowDialog()) return;
                file = new FileInfo(dlg.FileName);
            }
            var formatter = new BinaryFormatter();
            try {
                using(var s = file.OpenRead())
                    _sheetlayout = (SheetLayout)formatter.Deserialize(s);
            } catch(IOException ex) {
                TerminateFileOpening();
                MessageBox.Show("ファイルの読み取りに失敗しました。\r\n\r\n" + ex.Message,
                                "IOException",
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            } catch(SerializationException ex) {
                TerminateFileOpening();
                MessageBox.Show("用紙レイアウトデータではありません。\r\n\r\n" + ex.Message,
                                "SerializationException",
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _sheetlayout.File = file;
            _setting.SheetLayoutFilePath = file.Directory.FullName;
            EnableMenusOnLoad();
        }
        private void _tsmPrintSheets_Click(object sender, EventArgs e) {
            if(_card == null) return;
            using(var dlg = new PrintDialog()) {
                dlg.UseEXDialog = true;
                dlg.Document = _prtDocument;
                if(DialogResult.Cancel == dlg.ShowDialog()) return;
            }
            _prtDocument.OriginAtMargins = false;
            _prtDocument.PrintPage += new PrintPageEventHandler(_prtDocument_PrintSheets);
            _prtDocument.Print();
        }
        private void _prtDocument_PrintSheets(object sender, PrintPageEventArgs e) {
            e.Graphics.PageUnit = GraphicsUnit.Millimeter;
            for(var i = 0; i < _sheetlayout.M; i++)
                for(var j = 0; j < _sheetlayout.N; j++) {
                    SetTranslation(i, j, e);
                    _card.BePrintedOn(e.Graphics);
                }
            _prtDocument.PrintPage -= new PrintPageEventHandler(_prtDocument_PrintSheets);
        }
        private void PrintACardWithAFrame(PrintPageEventArgs e) {
            _card.BePrintedOn(e.Graphics);
            using(var pen = new Pen(_card.NegativeColor, 0))
                e.Graphics.DrawRectangle(pen, 0, 0, _card.Sizemm.Width, _card.Sizemm.Height);

        }
        private void SetTranslation(int i, int j, PrintPageEventArgs e) {
            e.Graphics.ResetTransform();
            var dx = i * _sheetlayout.IntervalHmm
                   + _sheetlayout.MarginLeftmm
                   - e.PageSettings.HardMarginX * MM_INCH100TH;
            var dy = j * _sheetlayout.IntervalVmm
                   + _sheetlayout.MarginTopmm
                   - e.PageSettings.HardMarginY * MM_INCH100TH;
            e.Graphics.TranslateTransform(dx, dy);
        }

        private void _tsmAbout_Click(object sender, EventArgs e) {
            const string format
                = "名刺エディタ Assembly version : {0}\r\n\r\n" +
                  "問合せ先 : 外池幸太郎\r\n" +
                  "e-mail : kt_dpmaster@hotmail.com\r\n" +
                  "URL : http://homepage3.nifty.com/numericworld/computer/misc/namecardeditor.htm\r\n\r\n" +
                  "Executable path : \r\n{1}\r\n\r\n" +
                  "Running on : .Net Framework {2}";
            var exepath = Application.ExecutablePath;
            var asm = Assembly.GetExecutingAssembly();
            var asmver = asm.GetName().Version;
            var netver = Environment.Version.ToString();
            MessageBox.Show(string.Format(format, asmver.ToString(), exepath, netver),
                            "名刺エディタについて",
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Information);
        }
    }
}